//
//  AuthorizedTasks.m
// added to CVS on 10/18/03
#import "AuthorizedTasks.h"
#import <Security/AuthorizationTags.h>

@implementation AuthorizedTasks

- init
{
    self = [super init];
    authorizationRef = NULL;
    return self;
}

- (void)dealloc
{
    [self deauthenticate];
    [super dealloc];
}

//============================================================================
//	- (BOOL)isAuthenticated
//============================================================================
// Find out if the user has the appropriate authorization rights.
// This really needs to be called each time you need to know whether the user
// is authorized, since the AuthorizationRef can be invalidated elsewhere, or
// may expire after a short period of time.
// As far as I know, there is no way to be notified when your AuthorizationRef
// expires.
//
- (BOOL)isAuthenticated
{
    AuthorizationRights rights;
    AuthorizationRights *authorizedRights;
    AuthorizationFlags flags;
    AuthorizationItem items[2];
    char *crontabWriterPath = "/usr/local/bin/psyncwriter";
    char *rmPath = "/bin/rm";	
    OSStatus err = 0;
    BOOL authorized = NO;
    
    if(authorizationRef==NULL)
    {
        //If we haven't created an AuthorizationRef yet, create one now with the
        //kAuthorizationFlagDefaults flags only to get the user's current
        //authorization rights.
        rights.count=0;
        rights.items = NULL;
        
        flags = kAuthorizationFlagDefaults;
	
        err = AuthorizationCreate(&rights,
				  kAuthorizationEmptyEnvironment, flags,
				  &authorizationRef);
        
    }
    
    //There should be one item in the AuthorizationItems array for each
    //type of right you want to acquire.
    //The data in the value and valueLength is dependant on which right you
    //want to acquire. For the right to execute tools as root,
    //kAuthorizationRightExecute, they should hold a pointer to a C string 
    //containing the path to the tool you want to execute, and 
    //the length of the C string path.
    //There needs to be one item for each tool you want to execute.
    items[0].name = kAuthorizationRightExecute;
    items[0].value = crontabWriterPath;
    items[0].valueLength = strlen(crontabWriterPath);
    items[0].flags = 0;
    items[1].name = kAuthorizationRightExecute;
    items[1].value = rmPath;
    items[1].valueLength = strlen(rmPath);
    items[1].flags = 0;
    
    rights.count=2;
    rights.items = items;
    
    flags = kAuthorizationFlagExtendRights;
    
    //Since we've specified kAuthorizationFlagExtendRights and
    //haven't specified kAuthorizationFlagInteractionAllowed, if the
    //user isn't currently authorized to execute tools as root,
    //they won't be asked for a password and err will indicate
    //an authorization failure.
    err = AuthorizationCopyRights(authorizationRef,&rights,
				  kAuthorizationEmptyEnvironment,
				  flags,&authorizedRights);
    
    authorized = (errAuthorizationSuccess==err);
    if(authorized)
    {
        //we don't need these items, and they need to be disposed of.
        AuthorizationFreeItemSet(authorizedRights);
    }
    return authorized;
}

//============================================================================
//	- (void)deauthenticate
//============================================================================
// Invalidate the user's AuthorizationRef to dispose of any acquired rights.
// It's a good idea to do this before you quit your application, else they
// will stay authorized as of the current time (4K78).
//
- (void)deauthenticate
{
    if(authorizationRef)
    {
        //dispose of any rights our AuthorizationRef has acquired, and null it out
        //so we get a new one next time we need one.
        AuthorizationFree(authorizationRef,kAuthorizationFlagDestroyRights);
        authorizationRef = NULL;
    }
}

//============================================================================
//	- (BOOL)fetchPassword
//============================================================================
// Fetch user's password if needed. If the user is already authorized, they
// will not be asked for their password again.
//
- (BOOL)fetchPassword
{
    AuthorizationRights rights;
    AuthorizationRights *authorizedRights;
    AuthorizationFlags flags;
    AuthorizationItem items[2];
    char *crontabWriterPath = "/usr/local/bin/psyncwriter";
    char *rmPath = "/bin/rm";
    OSStatus err = 0;
    BOOL authorized = NO;
    
    items[0].name = kAuthorizationRightExecute;
    items[0].value = crontabWriterPath;
    items[0].valueLength = strlen(crontabWriterPath);
    items[0].flags = 0;
    items[1].name = kAuthorizationRightExecute;
    items[1].value = rmPath;
    items[1].valueLength = strlen(rmPath);
    items[1].flags = 0;
    
    rights.count = 2;
    rights.items = items;
    
    flags = kAuthorizationFlagInteractionAllowed 
	| kAuthorizationFlagExtendRights;
    
    //Here, since we've specified kAuthorizationFlagExtendRights and
    //have also specified kAuthorizationFlagInteractionAllowed, if the
    //user isn't currently authorized to execute tools as root 
    //(kAuthorizationRightExecute),they will be asked for their password. 
    //The err return value will indicate authorization success or failure.
    err = AuthorizationCopyRights(authorizationRef,&rights,
				  kAuthorizationEmptyEnvironment,
				  flags,&authorizedRights);
    authorized = (errAuthorizationSuccess==err);
    if(authorized)
    {
        AuthorizationFreeItemSet(authorizedRights);
    }                                                    
    return authorized;
    
}


//============================================================================
//	- (BOOL)authenticate
//============================================================================
// if the user is not already authorized, ask them to authenticate.
// return YES if the user is (or becomes) authorized.
//
- (BOOL)authenticate
{
    if(![self isAuthenticated])
    {
        return [self fetchPassword]; //added return
    }
    return [self isAuthenticated];
}

// Updates our crontab file with our new copy
-(BOOL)authenticatedCronWriter
{
    char* args[1];
    OSStatus err = 0;
    
    if(![self authenticate])
        return FALSE;
    
    //the arguments parameter to AuthorizationExecuteWithPrivileges is
    //a NULL terminated array of C string pointers.
    args[0] = NULL;
    
    err = AuthorizationExecuteWithPrivileges(authorizationRef,"/usr/local/bin/psyncwriter", 0, args, NULL);
    if(err!=0)
    {
        NSBeep();
        NSLog(@"Couldn't find the psyncwriter helper tool in /usr/local/bin/",err);
		return FALSE;
    }
	return TRUE;
}

- (void)authenticatedLogDeleter
{
    char* args[3];
    OSStatus err = 0;
    
    if(![self authenticate])
	return;
    
    //the arguments parameter to AuthorizationExecuteWithPrivileges is
    //a NULL terminated array of C string pointers.
    args[0] = "-f";
    args[1] = "/var/log/psync.log";
    args[2] = NULL;
    
    err = AuthorizationExecuteWithPrivileges(authorizationRef,
					     "/bin/rm",
					     0, args, NULL);
    if(err!=0)
    {
	NSBeep();
	NSLog(@"Error %d in AuthorizationExecuteWithPrivileges",err);
    }
}

@end