// added to CVS on 10/18/03

#import "Controller.h"
#import "CronController.h";
#import "CronJob.h"
#import "AuthorizedTasks.h"
#import <Security/AuthorizationTags.h>

#define PSYNC_APPLESCRIPT [[[NSBundle bundleForClass:[self class]] resourcePath] stringByAppendingPathComponent:@"psyncapplescript"]

NSString *PSXDonationAlert=@"DonationSheetDisplayed";
NSString *PSXDontShowDonationAlert=@"DontShowDonationSheet";
NSString *PSXNeverShowDeleteFlagAlert=@"ShowDeleteFlagAlert";
NSString *PSXServerToCheck=@"ServerToCheck";

@implementation Controller

+(void)initialize
{

}

-(void)awakeFromNib
{
    // Setup the check boxes
    
    NSButtonCell *cell = [[NSButtonCell alloc] init];
    [cell setButtonType:NSSwitchButton];
    [cell setTitle:@""];
    [cell setImagePosition:NSImageOverlaps];
    [cell setControlSize:NSSmallControlSize];
    [[tableView tableColumnWithIdentifier:@"enabledDisabled"] setDataCell:cell]; // this set's the cell up to talk to our object directly.  We don't need to setup an action for it to call
    [cell release];
    
	
	// Setup our default preferences
	NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
	
	if ( [prefs objectForKey:PSXServerToCheck] == nil ) {
		[prefs setObject:[NSString stringWithString:@"http://psyncx.sourceforge.net/version.xml"] forKey:PSXServerToCheck];
	}
	if ( [prefs integerForKey:PSXDonationAlert] == nil ) {
		[prefs setInteger:0 forKey:PSXDonationAlert];
	}
	if ( [prefs boolForKey:PSXNeverShowDeleteFlagAlert] == nil ) {
		[prefs setBool:NO forKey:PSXNeverShowDeleteFlagAlert];
	}
    
    cronJobs=[[CronController alloc] init];
	
    // Then, we get our window position setup to update automatically
    [mainWindow setFrameAutosaveName:@"MainWindow"];
    
    if([tableView numberOfRows]>0) {
		[tableView selectRow:0 byExtendingSelection:FALSE];
    }
	[self updateInterface];
	[mainWindow makeFirstResponder:tableView];
}

-(void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    if([[NSUserDefaults standardUserDefaults] boolForKey:PSXDontShowDonationAlert]==NO) {
		SEL sel=@selector(_donationSheetDidEnd:returnCode:contextInfo:);
		NSString *tempStr=[NSString stringWithString:NSLocalizedString(@"DonationRequest", "Message that appears in the sheet asking for money")];
		
		// Don't want to do this, but I'm not sure how else to set the cancel button to nil
		if([[NSUserDefaults standardUserDefaults] integerForKey:PSXDonationAlert]>1) {
			// Run an alert panel
			NSBeginAlertSheet(NSLocalizedString(@"DonationRequestTitle", "Title for message that appears in the sheet asking for money"), //title
							  NSLocalizedString(@"DonationRequestDonateButton", "Title for donation button"),         //default button label
							  NSLocalizedString(@"DonationRequestNotNowButton", "Title for no donation button"),          //cancel button label
							  NSLocalizedString(@"DonationRequestStopBuggingMe", "Title for stop requesting donations button"),           //other button label
							  [tableView window],        //document window
							  self,           //modal delegate
							  sel,   //selector to method
							  NULL,          //dismiss selector
							  nil,           //context info
							  tempStr); // message
		}
		else {
			NSBeginAlertSheet(NSLocalizedString(@"DonationRequestTitle", "Title for message that appears in the sheet asking for money"), //title
							  NSLocalizedString(@"DonationRequestDonateButton", "Title for donation button"),         //default button label
							  NSLocalizedString(@"DonationRequestNotNowButton", "Title for no donation button"),          //cancel button label
							  nil,           //other button label
							  [tableView window],        //document window
							  self,           //modal delegate
							  sel,   //selector to method
							  NULL,          //dismiss selector
							  nil,           //context info
							  tempStr); // message
		}
    }
}

-(int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [cronJobs numberOfJobs];
}

-(id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    
    // What is the identifier for the column (ie, the column we're trying to fill?
    NSString *identifier =[aTableColumn identifier];
    // Which cron job?
    CronJob *cronObject=[cronJobs jobAtIndex:rowIndex];
    // What is the value from our object for the named column?
    return [cronObject valueForKey:identifier];
}

// this function allows our table to update our objects
-(void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    // This is equivalent to the code below
    [[cronJobs jobAtIndex:rowIndex] takeValue:anObject forKey:[aTableColumn identifier]];
    
	if([[aTableColumn identifier] isEqualToString:@"enabledDisabled"])
	{
		[self updateInterface];
	}
    // Name of the column
    //NSString *identifier=[aTableColumn identifier];
    
    // get the current object
    // CronJob *cronJob=[cronJobs jobAtIndex:rowIndex];
    
    // set the current object's key to our value
    //[cronJob takeValue:anObject forKey:identifier];
}

// This action is run when the Cancel button is clicked in a sheet
-(IBAction)closeSheet:(id)sender
{
    [self _closeASheet];
}

-(IBAction)openNewBackupSheet:(id)sender
{
    jobType=TRUE;
    [self _setupBackupWindow];
}

-(IBAction)openExistingBackupSheet:(id)sender
{
    jobType=FALSE;
    // If there is a row selected, let's load the info for it
    if([tableView selectedRow] != -1) {
	// Now we setup our window so that it contains all the
	// information out of our object
	[self _setupBackupWindow];
    }
    // If no selected row, we alert the user to the fact
    else {
	// No row selected
	NSBeginAlertSheet(NSLocalizedString(@"HangOn","No selected backup alert"), //title
			  NSLocalizedString(@"OK",nil),         //default button label
			  nil,           //cancel button label
			  nil,           //other button label
			  [tableView window],        //document window
			  nil,           //modal delegate
			  NULL,          //selector to method
			  NULL,          //dismiss selector
			  nil,           //context info
			  NSLocalizedString(@"SelectBackupToEdit",nil)); // message
    }
}

-(IBAction)deleteFlagAlert:(id)sender
{
    if([sender state] == NSOnState && [[NSUserDefaults standardUserDefaults] boolForKey:PSXNeverShowDeleteFlagAlert]==0) {
	NSString *temp=[NSString stringWithFormat:NSLocalizedString(@"DeleteOptionChecked","Explanation of what happens when delete option is checked"), [backupTo stringValue]];
	// Set our box to off
	[deleteFlagCheckBox setState:NSOffState];
	[deleteFlagAlertText setStringValue:temp];
	
	[NSApp beginSheet:deleteFlagAlertWindow
	   modalForWindow:backupWindow
	    modalDelegate:nil
	   didEndSelector:nil
	      contextInfo:nil];
	[NSApp runModalForWindow:deleteFlagAlertWindow];
	
	[NSApp endSheet:deleteFlagAlertWindow];
	[deleteFlagAlertWindow orderOut:self];
	[backupWindow makeKeyAndOrderFront:nil];
    }	
}
-(IBAction)deleteFlagAlertContinue:(id)sender
{
    if([deleteFlagCheckBox state] == NSOnState) {
	[[NSUserDefaults standardUserDefaults] setBool:[deleteFlagCheckBox state] forKey:PSXNeverShowDeleteFlagAlert];
    }	
    [self _closeASheet];
}
-(IBAction)deleteFlagAlertCancel:(id)sender
{
    [backupDeleteNonExistantFiles setState:NSOffState];
    [self _closeASheet];
}

-(IBAction)saveBackupSheet:(id)sender
{
    CronJob *job;
    NSString *temp;
    if([[backupFrom stringValue] isEqualToString:[backupTo stringValue]]) {
	NSRunAlertPanel(NSLocalizedString(@"BackupToSameFolderAlertTitle",nil),NSLocalizedString(@"BackupToSameFolderAlertText",nil),NSLocalizedString(@"OK", nil),nil,nil);
    }
    else {
	if(jobType == TRUE) {
	    // This is a new backup, so let's create a cron job
	    job=[[CronJob alloc] init];
	}
	else { // this is an existing job
	    job=[cronJobs jobAtIndex:[tableView selectedRow]];
	}
	
	// Set enabled/disabled
    [job setEnabledDisabled:[backupScheduleToggle state]];
	
	// Set the time from our menus
	if([backupMinuteMenu indexOfSelectedItem] == 12) {
	    temp=[NSString stringWithString:@"*"];
	}
	else {
	    temp=[NSString stringWithFormat:@"%i",[backupMinuteMenu indexOfSelectedItem] * 5];
	}
	
	[job setMinutes:temp];
	
	if([backupHourMenu indexOfSelectedItem] == 24) {
	    temp=[NSString stringWithString:@"*"];
	}
	else {
	    temp=[NSString stringWithFormat:@"%i",[backupHourMenu indexOfSelectedItem]];
	}
	
	[job setHours:temp];
	
	if([backupDayMenu indexOfSelectedItem] == 0) {
	    temp=[NSString stringWithString:@"*"];
	}
	else {
	    temp=[NSString stringWithFormat:@"%i",[backupDayMenu indexOfSelectedItem]];
	}
	
	[job setWeekDay:temp];
	
	[job setFlag:@"f" toState:[backupForceCopy state]];
	[job setFlag:@"d" toState:[backupDeleteNonExistantFiles state]];
	[job setFlag:@"r" toState:[backupRemoteBackup state]];
	[job setRunAsRootState:[backupRunAsRoot state]];
	[job setFrom:[backupFrom stringValue]];
	[job setTo:[backupTo stringValue]];
	
	if(jobType==TRUE) { // this is a new backup, so we add it to our array
	    [cronJobs addJob:job];
		[job autorelease];
	}
	
	// get the index back so we can select the appropriate row below
	int i=[cronJobs indexOfJob:job];

	// Refresh our table
	[tableView reloadData];
	
	// and set our gui to display correctly
	[self updateInterface];
	[tableView selectRow:i byExtendingSelection:FALSE];
	[mainWindow makeFirstResponder:tableView];
	
	[self _closeASheet];
    } // end if that checks to see if from and to are the same
}

-(IBAction)saveChanges:(id)sender
{
    if([cronJobs writeJobs])
	{
		[self updateInterface];
	}
}


-(IBAction)runBackupNow:(id)sender
{
    if([tableView numberOfSelectedRows]>1) {
	NSBeginAlertSheet(NSLocalizedString(@"HangOn", nil), //title
			  NSLocalizedString(@"OK", nil),         //default button label
			  nil,           //cancel button label
			  nil,           //other button label
			  [tableView window],        //document window
			  nil,           //modal delegate
			  NULL,          //selector to method
			  NULL,          //dismiss selector
			  nil,           //context info
			  NSLocalizedString(@"OneBackupAtATime",nil)); // message
    }
    else if([tableView numberOfSelectedRows]==0) {
	NSBeginAlertSheet(NSLocalizedString(@"HangOn", nil), //title
			  NSLocalizedString(@"OK", nil),         //default button label
			  nil,           //cancel button label
			  nil,           //other button label
			  [tableView window],        //document window
			  nil,           //modal delegate
			  NULL,          //selector to method
			  NULL,          //dismiss selector
			  nil,           //context info
			  NSLocalizedString(@"SelectBackupToRun",nil)); // message        
    }
    else {
	NSMutableArray *args=[NSMutableArray array];
	NSTask *aTask = [[NSTask alloc] init];
	
	NSString *tempString=[NSString stringWithFormat:@"%@", [[cronJobs jobAtIndex:[tableView selectedRow]] regularJob]];
	
	[args addObject:tempString];
	
	[aTask setLaunchPath:PSYNC_APPLESCRIPT];
	
	[aTask setArguments:args];
	[aTask launch];
    }
}

-(IBAction)backupFromSetter:(id)sender
{
    NSString *pathToFolder;
    NSOpenPanel *backupFromHere = [NSOpenPanel openPanel];
    [backupFromHere setAllowsMultipleSelection:NO];
    [backupFromHere setCanChooseDirectories:YES];
    [backupFromHere setCanChooseFiles:NO];
    [backupFromHere setPrompt:NSLocalizedString(@"From", nil)];
    [backupFromHere _setIncludeNewFolderButton:YES];
    
    if([[backupFrom stringValue] length] > 0) {
	pathToFolder=[NSString stringWithString:[backupFrom stringValue]];
    }
    else {
	pathToFolder=[NSString stringWithString:@"~"];
    }
    
    [backupFromHere beginSheetForDirectory:pathToFolder
				      file:nil
				     types:nil
			    modalForWindow:backupWindow
			     modalDelegate: self
			    didEndSelector:
	@selector(pathChosenForBackupFrom:
		  withReturn:
		  contextInfo:)
			       contextInfo: nil];
}

- (void)pathChosenForBackupFrom: (NSOpenPanel *) sheet withReturn:(int)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode == NSOKButton) {
	[backupFrom setStringValue:[sheet directory]];
    }
}

-(IBAction)backupToSetter:(id)sender
{
    NSString *pathToFolder;
    NSOpenPanel *backupToHere = [NSOpenPanel openPanel];
    [backupToHere setAllowsMultipleSelection:NO];
    [backupToHere setCanChooseDirectories:YES];
    [backupToHere setCanChooseFiles:NO];
    [backupToHere setPrompt:NSLocalizedString(@"To", nil)];
    [backupToHere _setIncludeNewFolderButton:YES];
    
    if([[backupTo stringValue] length] > 0) {
	pathToFolder=[NSString stringWithString:[backupTo stringValue]];
    }
    else {
	pathToFolder=[NSString stringWithString:@"~"];
    }
    
    [backupToHere beginSheetForDirectory:pathToFolder
				    file:nil
				   types:nil
			  modalForWindow:backupWindow
			   modalDelegate: self
			  didEndSelector:
	@selector(pathChosenForbackupTo:
		  withReturn:
		  contextInfo:)
			     contextInfo: nil];
}

- (void)pathChosenForbackupTo: (NSOpenPanel *) sheet withReturn:(int)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode == NSOKButton) {
	[backupTo setStringValue:[sheet directory]];
    }
}

-(IBAction)toggleEnabledDisabled:(id)sender;
{
    if ([sender state] == NSOnState ) {
        [backupMinuteMenu setEnabled:YES];
        [backupHourMenu setEnabled:YES];
        [backupDayMenu setEnabled:YES];
    }
    else {
        [backupMinuteMenu setEnabled:NO];
        [backupHourMenu setEnabled:NO];
        [backupDayMenu setEnabled:NO];
    }
}

- (IBAction)deleteBackup:(id)sender
{
    SEL sel=@selector(sheetDidEnd:returnCode:contextInfo:);
    
    int row =[tableView selectedRow];
    
    NSMutableString *tempStr;
    if(row !=-1)
    {
	// figure out whether this is singular or plural
	if ([tableView numberOfSelectedRows] > 1) {
	    tempStr=[NSString stringWithString:NSLocalizedString(@"CheckAboutDeletingRecords", nil)];
	}
	else
	{
	    tempStr=[NSString stringWithString:NSLocalizedString(@"CheckAboutDeletingOneRecord", nil)];
	}
	// Run an alert panel
	NSBeginAlertSheet(NSLocalizedString(@"HangOn", nil), //title
			  NSLocalizedString(@"Yes", nil),         //default button label
			  NSLocalizedString(@"No", nil),          //cancel button label
			  nil,    //other button label
			  [tableView window],        //document window
			  self,           //modal delegate
			  sel,   //selector to method
			  NULL,          //dismiss selector
			  nil,           //context info
			  tempStr); // message
    }
    else {
	// No row selected
	NSBeginAlertSheet(NSLocalizedString(@"HangOn", nil), //title
			  NSLocalizedString(@"OK", nil),         //default button label
			  nil,           //cancel button label
			  nil,           //other button label
			  [tableView window],        //document window
			  nil,           //modal delegate
			  NULL,          //selector to method
			  NULL,          //dismiss selector
			  nil,           //context info
			  NSLocalizedString(@"SelectARowBeforeICanDeleteIt", nil)); // message
	NSBeep();
    }
}
-(void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    //[self _closeASheet];
    if (returnCode==NSAlertDefaultReturn)
    {
	CronJob *currentJob;
	NSNumber *index;
	
	// Create an autoreleased mutable array
	NSMutableArray *jobsToRemove=[NSMutableArray array];
	
	// Get an enumerator for the selected rows
	NSEnumerator *e=[tableView selectedRowEnumerator];
	
	// Iterate over the selected rows
	while (index=[e nextObject])
	{
	    currentJob=[cronJobs jobAtIndex:[index intValue]];
	    [jobsToRemove addObject:currentJob];
	    //[cronJobs removeJob:currentJob];
	    //[currentJob release];
	}
	
	[cronJobs removeJobs:jobsToRemove];
	
	// Don't like to do this, but it crashes otherwise
	//		[jobsToRemove release];
	[tableView reloadData];
    }
}

-(IBAction)showLogWindow:(id)sender
{
    NSString *logString=[NSString stringWithContentsOfFile:@"/var/log/psync.log"];
    NSFont *font=[NSFont fontWithName:@"Courier" size:12];
    [backupLogWindow setFrameAutosaveName:@"LogWindow"];
    [backupLogWindow orderFront:self];
    [textView setFont:font];
    [textView setString:logString];
}

-(IBAction)deleteLog:(id)sender
{
    AuthorizedTasks *deleteLogFile=[[AuthorizedTasks alloc] init];
    [textView setString:@" "];
    [deleteLogFile authenticatedLogDeleter];
    [deleteLogFile release];
}

-(IBAction)checkVersion:(id)sender
{
    //	NSDictionary *myDict=[NSDictionary dictionaryWithContentsOfURL:[NSURL URLWithString:@"http://www.acornwebdesigns.com/psyncx/version.xml"]];
    NSDictionary *myDict=[NSDictionary dictionaryWithContentsOfURL:[NSURL URLWithString:[[NSUserDefaults standardUserDefaults] stringForKey:PSXServerToCheck]]];
    
    NSString *currVersionNumber = [[[NSBundle bundleForClass:[self class]] infoDictionary] objectForKey:@"CFBundleVersion"];
    
    NSString *latestVersionNumber = [myDict valueForKey:@"psync"];
    
    if(!([[myDict valueForKey:@"fileToCheck"] isEqualToString:@"0"])) {
        // if we don't have a zero, then save the new server location
		[[NSUserDefaults standardUserDefaults] setObject:[myDict valueForKey:@"fileToCheck"] forKey:PSXServerToCheck];
    }
    
    if(latestVersionNumber ==0) { //We weren't able to connect to the 'Net
	NSBeginAlertSheet(NSLocalizedString(@"AlertCantConnectToServer", nil), //title
			  NSLocalizedString(@"OK", nil),         //default button label
			  nil,           //cancel button label
			  nil,           //other button label
			  [tableView window],        //document window
			  nil,           //modal delegate
			  NULL,          //selector to method
			  NULL,          //dismiss selector
			  nil,           //context info
			  NSLocalizedString(@"AlertCantConnectToServerText", nil)); // message
    }
    else { // on the network
		if([currVersionNumber isEqualToString:latestVersionNumber]) {
			// no update
			NSBeginAlertSheet(NSLocalizedString(@"AlertNoUpdateAvailable", nil), //title
					  NSLocalizedString(@"OK", nil),         //default button label
					  nil,           //cancel button label
					  nil,           //other button label
					  [tableView window],        //document window
					  nil,           //modal delegate
					  NULL,          //selector to method
					  NULL,          //dismiss selector
					  nil,           //context info
					  NSLocalizedString(@"AlertUpToDate", nil)); // message
		}
		else { // new update available
			SEL sel=@selector(updateSheetDidEnd:returnCode:contextInfo:);
			NSBeginAlertSheet(NSLocalizedString(@"AlertNewUpDateAvailable", nil), //title
					  NSLocalizedString(@"Yes", nil),         //default button label
					  NSLocalizedString(@"No", nil),          //cancel button label
					  nil,    //other button label
					  [tableView window],        //document window
					  self,           //modal delegate
					  sel,   //selector to method
					  NULL,          //dismiss selector
					  (void *)[[NSString stringWithString:[myDict valueForKey:@"downloadURL"]] retain],           //context info
					  NSLocalizedString(@"AlertNewUpDateText", nil)); // message
		}
    }
}

// Example of how to use contextInfo here (see above as well)
-(void)updateSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:(NSString *)contextInfo]];
    }
    [(NSString *)contextInfo release];
}

-(IBAction)sendFeedback:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"mailto:psyncx@users.sourceforge.net?subject=PsyncX%20Feedback"]];
}

-(IBAction)showHelp:(id)sender
{
    [[NSWorkspace sharedWorkspace] openFile:[[NSBundle mainBundle] pathForResource:@"ReadMe" ofType:@"rtfd"]];
}

-(IBAction)donateMoney:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"https://www.paypal.com/xclick/business=paypal%40acorncomputersolutions.com&item_name=Donate+To+PsyncX+Fund(written+by+Acorn+Computer+Solutions)&no_shipping=1&cn=Optional+Feedback&tax=0&currency_code=USD"]];
}

-(BOOL)validateMenuItem:(NSMenuItem *)item {
	int numOfRows=[tableView numberOfSelectedRows];
	// if it is a menu item related to our backup items and we don't have any, then let's disable them
	if (([item action] == @selector(openExistingBackupSheet:) || [item action] == @selector(deleteBackup:) || [item action] == @selector(runBackupNow:)) && numOfRows==0) {
        return NO;
    }
	if([item action] == @selector(saveChanges:) && ![cronJobs needToSave])
	{
		return NO;
	}
	return YES;
}

-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	[self updateInterface];
}



-(BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)app
{
    return YES;
}
/*
 static int _moveRow = 0;
 
 // We want to allow drag and drop
 - (BOOL)tableView:(NSTableView *)tv writeRows:(NSArray*)rows toPasteboard:(NSPasteboard*)pboard
 {
     // If we only have one job, we shouldn't bother with this
     //	NSString *type = [tableView autosaveName];
     NSString *type = [NSString stringWithString:@"CronJobPasteBoard"];
     if ([cronJobs numberOfJobs] < 2)
     {
	 return NO;
     }
     [pboard declareTypes:[NSArray arrayWithObject:type] owner:self];
     //    [pboard setPropertyList:rows forType:type];
     
     //	[pboard declareTypes:[NSArray arrayWithObjects:type] owner:self];
     [pboard setData:[NSArchiver archivedDataWithRootObject:rows] forType:type];
     //[pboard setString:[rows description] forType: NSStringPboardType];
     if ([rows count] == 1) _moveRow = [[rows objectAtIndex:0]intValue];
     NSLog(@"here: %@", type);
     return YES;
 }
 
 
 - (NSDragOperation)tableView:(NSTableView*)tableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation
 {
     return NSDragOperationMove;
 }
 
 /*- (unsigned int)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
 // This method is used by NSTableView to determine a valid drop target. Based on the mouse position, the table view will suggest a proposed drop location. This method must return a value that indicates which dragging operation the data source will perform. The data source may "re-target" a drop if desired by calling setDropRow:dropOperation: and returning something other than NSDragOperationNone. One may choose to re-target for various reasons (eg. for better visual feedback when inserting into a sorted position).
 {
     if (row != _moveRow) {
	 if (op==NSTableViewDropAbove /*NSTableViewDropOn*//*) {
	     return NSDragOperationAll;
	 }
	 return NSDragOperationNone;
     }
 return NSDragOperationNone;
 }
 - (BOOL)tableView:(NSTableView*)tableView acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation
 {
     /*    NSArray *selectedRows;
     NSEnumerator *rowEnum;
     NSNumber *aRow;
     
     //    if(row<0)
     //      return NO;
     NSLog(@"Dropping at row: %d", row);
     
     selectedRows = [NSUnarchiver unarchiveObjectWithData:[[info draggingPasteboard] dataForType:[tableView autosaveName]]];
     
     NSLog(@"test");
     rowEnum = [selectedRows objectEnumerator];
     
     while(aRow = [rowEnum nextObject])
 {
	 int index = [aRow intValue];
	 
	 id obj = [[self objectAtIndex:index] retain];
	 [self replaceObjectAtIndex:index withObject:[NSNull null]];
	 
	 [self insertObject:obj atIndex:row];
	 [self removeObject:[NSNull null]];
 }
     
     [tableView reloadData];
     return YES;
 }
 */
     
- (void)dealloc
 {
	[[NSUserDefaults standardUserDefaults] synchronize];
     [cronJobs release];
     [super dealloc];
 }
@end

@implementation Controller (Private)
-(void)updateInterface
{
	BOOL onOff;
    if([tableView numberOfSelectedRows]>0) {
		onOff=TRUE;
	}
	else {
		onOff=FALSE;
	}
    [editBackup setEnabled:onOff];
    [deleteBackup setEnabled:onOff];
    [runBackup setEnabled:onOff];
	
	if([cronJobs needToSave])
	{
		[backupOkayButton setKeyEquivalent:@""];
		[newBackup setKeyEquivalent:@""];
		[saveBackup setKeyEquivalent:@"\r"];		
		[saveBackup setEnabled:TRUE];
	}
	else
	{
		[backupOkayButton setKeyEquivalent:@""];
		[newBackup setKeyEquivalent:@"\r"];
		[saveBackup setKeyEquivalent:@""];		
		[saveBackup setEnabled:FALSE];
	}
}

-(void)_donationSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if (returnCode==NSAlertDefaultReturn) // they want to donate something
    {
        [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"https://www.paypal.com/xclick/business=paypal%40acorncomputersolutions.com&item_name=Donate+To+PsyncX+Fund(written+by+Acorn+Computer+Solutions)&no_shipping=1&cn=Optional+Feedback&tax=0&currency_code=USD"]];
    }
    if (returnCode==NSAlertOtherReturn) // they don't want to see this alert again
    {
		[[NSUserDefaults standardUserDefaults] setBool:YES forKey:PSXDontShowDonationAlert];
    }
    // We're still bugging them, increment our sheet showing counter.  After we get to 2, we don't need to increment any more
    if([[NSUserDefaults standardUserDefaults] integerForKey:PSXDonationAlert]<2) {
		[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:[[NSUserDefaults standardUserDefaults] integerForKey:PSXDonationAlert]+1] forKey:PSXDonationAlert];
    }
}

-(void)_resetBackupWindow:(NSString *)temp
{
    [backupFrom setStringValue:@""];
    [backupTo setStringValue:@""];
    
    [backupForceCopy setState:NSOffState];
    [backupDeleteNonExistantFiles setState:NSOffState];
    [backupRemoteBackup setState:NSOffState];
    
    [backupMinuteMenu selectItemAtIndex:0];
    [backupHourMenu selectItemAtIndex:0];
    [backupDayMenu selectItemAtIndex:0];
    
    // Should be able to combine this with my toggleEnabledDisabled code
    if([temp isEqualToString:@"#"])
    {
		// Turn the toggle off and all the menus
		[backupScheduleToggle setState:NSOffState];
		[backupMinuteMenu setEnabled:NO];
		[backupHourMenu setEnabled:NO];
		[backupDayMenu setEnabled:NO];
    }
    else
    {
		[backupScheduleToggle setState:NSOnState];
		[backupMinuteMenu setEnabled:YES];
		[backupHourMenu setEnabled:YES];
		[backupDayMenu setEnabled:YES];
    }
}

-(void)_setupBackupWindow
{
    NSString *backupType;
    NSString *windowTitle;
    NSString *buttonTitle;
    CronJob *existingBackup;
    
    if(jobType == TRUE) {
		backupType=[NSString stringWithString:@""];
		windowTitle=[NSString stringWithString:NSLocalizedString(@"NewBackupWindowTitle",nil)];
		buttonTitle=[NSString stringWithString:NSLocalizedString(@"NewBackupButtonTitle",nil)];
    }
    else {
		existingBackup=[cronJobs jobAtIndex:[tableView selectedRow]];
		
		backupType=[NSString stringWithString:[existingBackup enabledDisabledString]];
		windowTitle=[NSString stringWithString:NSLocalizedString(@"EditBackupWindowTitle",nil)];
		buttonTitle=[NSString stringWithString:NSLocalizedString(@"EditBackupButtonTitle",nil)];
    }
    
	[backupOkayButton setKeyEquivalent:@"\r"];
    // We then set the items using our variables from above
    [self _resetBackupWindow:backupType];
    [backupWindow setTitle:windowTitle];
    [backupOkayButton setTitle:buttonTitle];
    [backupWindow makeFirstResponder:backupFrom];
    
    
    if(jobType==FALSE) { // have an existing job
		
		[backupFrom setStringValue:[existingBackup from]];
		[backupTo setStringValue:[existingBackup to]];
		
		[backupForceCopy setState:[existingBackup fFlagState]];
		[backupDeleteNonExistantFiles setState:[existingBackup dFlagState]];
		[backupRemoteBackup setState:[existingBackup rFlagState]];
		[backupRunAsRoot setState:[existingBackup runAsRootState]];
		
		if([[existingBackup hours] isEqualToString:@"*"]) {
			[backupHourMenu selectItemAtIndex:24];
		}
		else {
			[backupHourMenu selectItemAtIndex:[[existingBackup hours] intValue]];
		}
		if([[existingBackup minutes] isEqualToString:@"*"]) {
			[backupMinuteMenu selectItemAtIndex:12];
		}
		else {
			[backupMinuteMenu selectItemAtIndex:[[existingBackup minutes] intValue]/5];
		}
		if([[existingBackup weekDay] isEqualToString:@"*"]) {
			[backupDayMenu selectItemAtIndex:0];
		}
		else if([[existingBackup weekDay] isEqualToString:@"0"]) {
			[backupDayMenu selectItemAtIndex:7];			
		}
		else {
			[backupDayMenu selectItemAtIndex:[[existingBackup weekDay] intValue]];
		}
    }
    //[existingBackup release];
    
    [NSApp beginSheet:backupWindow
       modalForWindow:mainWindow
		modalDelegate:nil
       didEndSelector:nil
		  contextInfo:nil];
    [NSApp runModalForWindow:backupWindow];
    
    [NSApp endSheet:backupWindow];
    [backupWindow orderOut:self];
    [mainWindow makeKeyAndOrderFront:nil];
}

-(void)_closeASheet
{
    [NSApp stopModal];
}
@end