//
//  CronController.m
//  PsyncX
//
//  Created by David Baker on Mon May 12 2003.
//  Copyright (c) 2003 AcornSoft. All rights reserved.
//
// added to CVS on 10/18/03

#import "CronController.h"
#import "CronJob.h"
#import "AuthorizedTasks.h"

@implementation CronController

-(id)init
{
    if(self = [super init]) {
		
		// Then fill the array
		[self readJobs];
		originalNumberOfJobs=[jobsArray count];
    }
    return self;
}

-(BOOL)needToSave;
{
	unsigned int i, count = [jobsArray count];
	if(originalNumberOfJobs!=count) // check this first so we can fall out faster
	{
		return TRUE;
	}
	for (i = 0; i < count; i++) {
		CronJob *job = [jobsArray objectAtIndex:i];
		if ([job needToSave]) { // if one of them needs to save, then we can return
			return TRUE;
		}
	}
	return FALSE;
}

-(void)readJobs
{
    // Start by declaring our variables
    int i=0;
    int j=0;
    int topLine=0;
    int bottomLine=0;
    
    
    // Read in /etc/crontab and
    // Put it in a string so we can put it in an array
    NSString *crontabString=[NSString stringWithContentsOfFile:@"/etc/crontab"];
    
    // then we split our file up
    NSArray *crontest = [NSArray arrayWithArray:[crontabString componentsSeparatedByString:@"\n"]];
    
    // next, check to see if we have our PsyncX lines
    
    for(i=0;i<[crontest count];i++) {
		// Check each item to see if it is one of our PsyncX comment lines
		NSString *temp=[NSString stringWithString:[crontest objectAtIndex:i]];
		if([temp hasPrefix:@"###PsyncX Cro"]) {
			topLine=i+1;
		}
		else if([temp hasPrefix:@"###End PsyncX"]) {
			bottomLine=i;
		}
		else {
			// Wait and see
		}
    }
    
    jobsArray=[[NSMutableArray alloc] init];
    
    // If we don't have any PsyncX comment lines
    // we need to do some conversions from the old way of doing things
    if (topLine==0 && bottomLine==0) {
		// Now that we're here, we're going to need to do a search to see
		// which lines have the string /usr/bin/runpsync in them
		for(i=0;i<[crontest count];i++) {
            // Check each item to see if it contains the string above
			
            // See Strings in Cocoa: Part I (Page 3) on Mac Dev Center for how this works
            NSString *temp=[NSString stringWithString:[crontest objectAtIndex:i]];
            NSRange range = [temp rangeOfString:@"/usr/local/bin/runpsync"];
            int length = range.length;
            // If our length is greater than zero, then we have a line with our text in it
            if(length>0) {
                CronJob *newObj=[[CronJob alloc] init];
				//check into how to
				// do the auto-releasing of this object
				
                [newObj parseOldJob:temp];
                [jobsArray addObject:newObj];
				[newObj release];
            }
        }
    }
    // We're doing things the new way
    else {
		
        // Need to do some fancy manipulation of the cron file, so that we have our
        // items at the bottom in general
		
        // Also, what happens if we don't find both lines?
		for(j=topLine; j<bottomLine; j++) {
            NSString *temp2=[NSString stringWithString:[crontest objectAtIndex:j]];
            CronJob *newObj=[[CronJob alloc] init]; //check into how to
													// do the auto-releasing of this object
			
            [newObj parseJob:temp2];
            [jobsArray addObject:newObj];
			[newObj release];
        }
    }
}

-(int)numberOfJobs
{
    return [jobsArray count];
}

-(CronJob *)jobAtIndex:(int)i
{
    return [jobsArray objectAtIndex:i];
}

-(void)addJob:(CronJob *)job;
{
	[jobsArray addObject:job];
}

-(int)indexOfJob:(CronJob *)job;
{
	return [jobsArray indexOfObject:job];	
}

-(BOOL)writeJobs
{
    // Declare our variables
    int i=0;
    int topLine=0;
    int bottomLine=0;
    AuthorizedTasks *rootWrite;
	BOOL returnValue;
    
    // Start by reading in our file
    // Put it in a string so we can put it in an array
    NSString *crontabString=[NSString stringWithContentsOfFile:@"/etc/crontab"];
    
    // then we split our file up
    NSMutableArray *crontest = [NSMutableArray arrayWithArray:[crontabString componentsSeparatedByString:@"\n"]];
    
    // Check to see if we have edited this file before
    // (this is indicated by the ###PsyncX Crontab### line)
    
    for(i=0;i<[crontest count];i++) {
		// Check each item to see if it is one of our PsyncX comment lines
		NSString *temp=[NSString stringWithString:[crontest objectAtIndex:i]];
		if([temp hasPrefix:@"###PsyncX Cro"]) {
			topLine=i+1;
		}
		else if([temp hasPrefix:@"###End PsyncX"]) {
			bottomLine=i-1;
		}
		else {
			// Wait and see
		}
    }
    
    // If we have comment lines then write our lines in between
    if (topLine>0 && bottomLine>0) {
		while(topLine!=bottomLine+1) {
			// Delete an item from our array
			[crontest removeObjectAtIndex:topLine];
			bottomLine--;
		}
		// Continue doing so until our topLine and bottomLine variables
		// are the same.  Then we can insert our cron items at our topLine
		// index
		
    }
    // If we don't have any PsyncX comment lines
    // we need to do some conversions from the old way of doing things
    else {
        for(i=0;i<[crontest count];i++) {
			// Check each item to see if it contains runpsync
            NSString *temp=[NSString stringWithString:[crontest objectAtIndex:i]];
            NSRange range = [temp rangeOfString:@"/usr/local/bin/runpsync"];
			
            int length = range.length;
			
			// If our length is greater than zero, then we have a line with our text in it
            if(length>0) {
                if(topLine==0) {
                    topLine=i;
                }
                bottomLine=i;
            }
            else {
                if(topLine>0) {
					// Select the object we've found that shouldn't be in between our commands
					NSString *temp=[NSString stringWithString:[crontest objectAtIndex:i]];
					[crontest removeObjectAtIndex:i];
					[crontest insertObject:temp atIndex:topLine];
					topLine+=1;
					bottomLine+=1;
                }
            }
            
			
		}
		
		if(topLine==0) { // we don't have any old jobs, let's set topLine to the bottom of the file so that we append to the file
			topLine=i+1;
		}
		else {
			// erase the old jobs
			while(topLine!=bottomLine+1) {
				// Delete an item from our array
				[crontest removeObjectAtIndex:topLine];
				bottomLine--;
			}
		}
		
		// Insert our comment lines so that we don't have to parse this the old way next time
		[crontest insertObject:[NSString stringWithString:@"\n###PsyncX Crontab###"] atIndex:topLine-1];
		[crontest insertObject:[NSString stringWithString:@"###End PsyncX Crontab###"] atIndex:topLine];
    }
    
    // Insert our new jobs into the list
    for(i=0;i<[jobsArray count];i++) {
		[crontest insertObject:[[jobsArray objectAtIndex:i] cronJob] atIndex:topLine+i];
		[[jobsArray objectAtIndex:i] setNeedToSave:FALSE];
    }
    
    [[crontest componentsJoinedByString:@"\n"] writeToFile:@"/tmp/crontab.txt" atomically:YES];
    
    rootWrite=[[AuthorizedTasks alloc] init];
    returnValue=[rootWrite authenticatedCronWriter];
    [rootWrite release];
	
	// we reset this so that we are accurate to what is in the cron file at the moment
	originalNumberOfJobs=[jobsArray count];
	return returnValue;
}

-(void)removeJobs:(NSMutableArray *)tempArray
{
    //	int i=0;
    [jobsArray removeObjectsInArray:tempArray];
    //	for(i=0;i<[tempArray count];i++) {
    //		[[tempArray objectAtIndex:i] release];
    //	}
}
-(void)removeJob:(CronJob *)job
{
    [jobsArray removeObject:job];
}

-(void)dealloc
{
    [jobsArray release];
    [super dealloc];
}
@end
